ThemeEntry

Enum ThemeEntry 

pub enum ThemeEntry {
    Ansi(AnsiColor),
    DefaultForeground(),
    DefaultBackground(),
}
A color theme entry.

This enumeration combines a variant wrapping an AnsiColor with two more variants for the default foreground and background colors to identify the ThemeEntry::COUNT entries of a color theme. Displaying a theme entry produces the ANSI escape sequence used to query a terminal for the corresponding color.

Variants



Ansi(AnsiColor)



DefaultForeground()



DefaultBackground()

Implementations

Source

impl ThemeEntry

pub const COUNT: usize = 18

The total number of theme entries.

pub fn all() -> ThemeEntryIterator

Create a new iterator over all theme entries in canonical order.

Source

impl ThemeEntry

pub fn name(&self) -> &'static str

Get this theme entry’s human-readable name.

pub fn abbr(&self) -> &'static str

Get an abbreviation for this theme entry’s name.

This method returns a two-letter abbreviations for this theme entry. See AnsiColor::abbr for a description of the abbreviations for ANSI colors.

Source

impl ThemeEntry

pub fn request(&self) -> RequestColor

Convert the theme entry to a color request. TTY only!