pub enum ThemeEntry {
Ansi(AnsiColor),
DefaultForeground(),
DefaultBackground(),
}Expand description
A color theme entry.
This enumeration combines a variant wrapping an AnsiColor with two more
variants for the default foreground and background colors to identify the
ThemeEntry::COUNT entries of a color theme. Displaying a theme entry
produces the ANSI escape sequence used to query a terminal for the
corresponding color.
Variants
Implementations
Sourceimpl ThemeEntry
impl ThemeEntry
Sourceimpl ThemeEntry
impl ThemeEntry
Sourcepub fn abbr(&self) -> &'static str
pub fn abbr(&self) -> &'static str
Get an abbreviation for this theme entry’s name.
This method returns a two-letter abbreviations for this theme entry. See
AnsiColor::abbr for a description of the abbreviations for ANSI
colors.
Sourceimpl ThemeEntry
impl ThemeEntry
Sourcepub fn request(&self) -> RequestColor
pub fn request(&self) -> RequestColor
Convert the theme entry to a color request. TTY only!